@@ -846,7 +846,9 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
                      align, align, lv, scale);
         if(anchor) tcxstrprintf(rbuf, "<a href=\"%@\">", url);
         if(object){
-          tcxstrprintf(rbuf, "<object data=\"%@\" type=\"%@\"", url, mimetype(url));
+          tcxstrprintf(rbuf, "<object data=\"%@\"", url);
+          const char *type = mimetype(url);
+          if(type) tcxstrprintf(rbuf, " type=\"%@\"", type);
           if(width > 0) tcxstrprintf(rbuf, " width=\"%d%@\"", width, wratio ? "%" : "");
           if(height > 0) tcxstrprintf(rbuf, " height=\"%d%@\"", height, hratio ? "%" : "");
           tcxstrprintf(rbuf, ">");
@@ -917,7 +919,8 @@ void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri){
           *sep = '\0';
           uri = sep + 1;
         }
-        if(tcstrfwm(uri, "http://") || tcstrfwm(uri, "https://")){
+        if(tcstrfwm(uri, "http://") || tcstrfwm(uri, "https://") ||
+           tcstrfwm(uri, "ftp://") || tcstrfwm(uri, "mailto:")){
           tcxstrprintf(rbuf, "<a href=\"%@\">", uri);
         } else if(tcstrfwm(uri, "id:")){
           int64_t id = tcatoi(tcstrskipspc(strchr(uri, ':') + 1));
@@ -1029,13 +1032,13 @@ char *datestrsimple(char *str){
 const char *mimetype(const char *name){
   assert(name);
   const char *list[] = {
-    "txt", "text/plain", "asc", "text/plain", "in", "text/plain", "c", "text/plain",
-    "h", "text/plain", "cc", "text/plain", "java", "text/plain", "sh", "text/plain",
-    "pl", "text/plain", "py", "text/plain", "rb", "text/plain", "idl", "text/plain",
-    "csv", "text/plain", "log", "text/plain", "conf", "text/plain", "rc", "text/plain",
-    "ini", "text/plain", "html", "text/html", "htm", "text/html", "xhtml", "text/html",
-    "xht", "text/html", "css", "text/css", "js", "text/javascript",
-    "tsv", "text/tab-separated-values",
+    "txt", "text/plain", "text", "text/plain", "asc", "text/plain", "in", "text/plain",
+    "c", "text/plain", "h", "text/plain", "cc", "text/plain", "java", "text/plain",
+    "sh", "text/plain", "pl", "text/plain", "py", "text/plain", "rb", "text/plain",
+    "idl", "text/plain", "csv", "text/plain", "log", "text/plain", "conf", "text/plain",
+    "rc", "text/plain", "ini", "text/plain",
+    "html", "text/html", "htm", "text/html", "xhtml", "text/html", "xht", "text/html",
+    "css", "text/css", "js", "text/javascript", "tsv", "text/tab-separated-values",
     "eml", "message/rfc822", "mime", "message/rfc822", "mht", "message/rfc822",
     "mhtml", "message/rfc822", "sgml", "application/sgml",
     "sgm", "application/sgml", "xml", "application/xml", "xsl", "application/xml",
@@ -1066,14 +1069,33 @@ const char *mimetype(const char *name){
     "avi", "video/x-msvideo",
     NULL
   };
-  const char *ext = strrchr(name, '.');
+  int len = strlen(name);
+  char stack[1024];
+  char *buf = NULL;
+  if(len < sizeof(stack)){
+    buf = stack;
+  } else {
+    buf = tcmalloc(len + 1);
+  }
+  memcpy(buf, name, len);
+  buf[len] = '\0';
+  char *pv = strchr(buf, '#');
+  if(pv) *pv = '\0';
+  pv = strchr(buf, '?');
+  if(pv) *pv = '\0';
+  const char *ext = strrchr(buf, '.');
+  const char *type = NULL;
   if(ext){
     ext++;
     for(int i = 0; list[i] != NULL; i++){
-      if(!tcstricmp(ext, list[i])) return list[i+1];
+      if(!tcstricmp(ext, list[i])){
+        type = list[i+1];
+        break;
+      }
     }
   }
-  return "application/octet-stream";
+  if(buf != stack) tcfree(buf);
+  return type;
 }
 
 
