@@ -50,7 +50,7 @@ __COMMON_CLINKAGEBEGIN
  *************************************************************************************************/
 
 
-#define TPVERSION      "0.9.6"
+#define TPVERSION      "0.9.7"
 
 #define TUNEBNUM       131071            // bnum tuning parameter
 #define TUNEAPOW       6                 // apow tuning parameter
@@ -130,7 +130,7 @@ char *datestrsimple(char *str);
 
 /* Get the MIME type of a file.
    `name' specifies the name of the file.
-   The return value is the MIME type of the file. */
+   The return value is the MIME type of the file or `NULL' if the type is not detected. */
 const char *mimetype(const char *name);
 
 
