@@ -36,6 +36,7 @@ int g_feedlistnum;                       // number of articles in a RSS feed
 int g_filenum;                           // number of files in a file list page
 const char *g_commentmode;               // comment mode
 const char *g_updatecmd;                 // path of the update command
+int g_sessionlife;                       // lifetime of each session
 const char *g_frontpage;                 // name of the front page
 
 
@@ -97,6 +98,8 @@ int main(int argc, char **argv){
     if(!g_commentmode) g_commentmode = "";
     g_updatecmd = tctmplconf(g_tmpl, "updatecmd");
     if(!g_updatecmd) g_updatecmd = "";
+    rp = tctmplconf(g_tmpl, "sessionlife");
+    g_sessionlife = tclmax(rp ? tcatoi(rp) : 0, 0);
     g_frontpage = tctmplconf(g_tmpl, "frontpage");
     if(!g_frontpage) g_frontpage = "";
     TCMAP *conf = g_tmpl->conf;
@@ -831,6 +834,7 @@ static void dosession(TCMPOOL *mpool){
       } else {
         tcmapput2(vars, "view", "single");
       }
+      tcmapput2(vars, "robots", "index,follow");
       tcmapputmap(vars, "art", cols);
     } else {
       tcmapput2(vars, "view", "empty");
@@ -873,6 +877,7 @@ static void dosession(TCMPOOL *mpool){
         } else {
           tcmapput2(vars, "view", "single");
         }
+        tcmapput2(vars, "robots", "index,follow");
         tcmapputmap(vars, "art", cols);
       } else {
         tcmapput2(vars, "view", "empty");
@@ -890,6 +895,7 @@ static void dosession(TCMPOOL *mpool){
       if(tclistnum(arts) > 0){
         tcmapprintf(vars, "titletip", "[name:%s]", p_name);
         tcmapput2(vars, "view", "search");
+        tcmapput2(vars, "robots", "noindex,follow");
         if(p_page > 1) tcmapprintf(vars, "prev", "%d", p_page - 1);
         if(rnum > max) tcmapprintf(vars, "next", "%d", p_page + 1);
         tcmapputlist(vars, "arts", arts);
@@ -911,28 +917,26 @@ static void dosession(TCMPOOL *mpool){
     }
     tcmapprintf(vars, "titletip", "[search]");
     tcmapput2(vars, "view", "search");
-    if(*p_cond != '\0'){
-      int max = g_searchnum;
-      int skip = max * (p_page - 1);
-      TCLIST *res = searcharts(mpool, tdb, p_cond, p_expr, p_order, max + 1, skip, true);
-      int rnum = tclistnum(res);
-      TCLIST *arts = tcmpoollistnew(mpool);
-      for(int i = 0; i < rnum && i < max; i++){
-        int64_t id = tcatoi(tclistval2(res, i));
-        TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
-        if(cols){
-          setarthtml(mpool, cols, id, 1, true);
-          tclistpushmap(arts, cols);
-        }
+    tcmapput2(vars, "robots", "noindex,follow");
+    int max = g_searchnum;
+    int skip = max * (p_page - 1);
+    TCLIST *res = searcharts(mpool, tdb, p_cond, p_expr, p_order, max + 1, skip, true);
+    int rnum = tclistnum(res);
+    TCLIST *arts = tcmpoollistnew(mpool);
+    for(int i = 0; i < rnum && i < max; i++){
+      int64_t id = tcatoi(tclistval2(res, i));
+      TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
+      if(cols){
+        setarthtml(mpool, cols, id, 1, true);
+        tclistpushmap(arts, cols);
       }
-      if(tclistnum(arts) > 0){
-        if(*p_expr != '\0') tcmapprintf(vars, "titletip", "[search:%s]", p_expr);
-        if(p_page > 1) tcmapprintf(vars, "prev", "%d", p_page - 1);
-        if(rnum > max) tcmapprintf(vars, "next", "%d", p_page + 1);
-        if(tcmapget2(vars, "prev") || tcmapget2(vars, "next")) tcmapput2(vars, "page", "true");
-        tcmapputlist(vars, "arts", arts);
-      }
-      tcmapprintf(vars, "resnum", "%d", rnum);
+    }
+    if(tclistnum(arts) > 0){
+      if(*p_expr != '\0') tcmapprintf(vars, "titletip", "[search:%s]", p_expr);
+      if(p_page > 1) tcmapprintf(vars, "prev", "%d", p_page - 1);
+      if(rnum > max) tcmapprintf(vars, "next", "%d", p_page + 1);
+      if(tcmapget2(vars, "prev") || tcmapget2(vars, "next")) tcmapput2(vars, "page", "true");
+      tcmapputlist(vars, "arts", arts);
     }
   } else if(*g_frontpage != '\0' && strcmp(p_act, "timeline")){
     // front view
@@ -959,6 +963,7 @@ static void dosession(TCMPOOL *mpool){
       if(tclistnum(res) > 0) id = tcatoi(tclistval2(res, 0));
     }
     tcmapput2(vars, "view", "front");
+    tcmapput2(vars, "robots", "index,follow");
     if(id > 0){
       TCMAP *cols = tcmpoolpushmap(mpool, dbgetart(tdb, id));
       if(cols){
@@ -968,7 +973,7 @@ static void dosession(TCMPOOL *mpool){
       }
     }
   } else {
-    // list view
+    // timeline view
     if(!auth){
       if(mtime <= p_ifmod){
         showcache();
@@ -1011,6 +1016,7 @@ static void dosession(TCMPOOL *mpool){
           }
         }
         tcmapput2(vars, "view", "timeline");
+        tcmapput2(vars, "robots", "index,follow");
         if(p_page > 1) tcmapprintf(vars, "prev", "%d", p_page - 1);
         if(rnum > max) tcmapprintf(vars, "next", "%d", p_page + 1);
         if(tcmapget2(vars, "prev") || tcmapget2(vars, "next")) tcmapput2(vars, "page", "true");
@@ -1051,22 +1057,25 @@ static void dosession(TCMPOOL *mpool){
   tcmapput2(vars, "hostname", p_hostname);
   tcmapput2(vars, "useragent", p_useragent);
   tcmapput2(vars, "userlang", p_userlang);
-  const char *mimetype = "text/html; charset=UTF-8";
+  const char *mtype = "text/html; charset=UTF-8";
   if(!strcmp(p_format, "atom")){
-    mimetype = "application/atom+xml";
+    mtype = "application/atom+xml";
   } else if(!strcmp(p_format, "xhtml")){
-    mimetype = "application/xhtml+xml";
+    mtype = "application/xhtml+xml";
   } else if(!strcmp(p_format, "xml")){
-    mimetype = "application/xml";
+    mtype = "application/xml";
   }
-  tcmapput2(vars, "mimetype", mimetype);
+  tcmapput2(vars, "mimetype", mtype);
   const char *tmplstr = tcmpoolpushptr(mpool, tctmpldump(g_tmpl, vars));
-  printf("Content-Type: %s\r\n", mimetype);
+  printf("Content-Type: %s\r\n", mtype);
   rp = tcmapget2(vars, "lastmod");
   if(rp) printf("Last-Modified: %s\r\n", rp);
   printf("Cache-Control: no-cache\r\n");
-  if(authcookie)
-    printf("Set-Cookie: auth=%s; max-age=%d; path=%s\r\n", authcookie, 3600 * 24, g_scriptname);
+  if(authcookie){
+    printf("Set-Cookie: auth=%s;", authcookie);
+    if(g_sessionlife > 0) printf(" max-age=%d;", g_sessionlife);
+    printf(" path=%s\r\n", g_scriptname);
+  }
   printf("\r\n");
   fwrite(tmplstr, 1, strlen(tmplstr), stdout);
   fflush(stdout);
@@ -1337,6 +1346,7 @@ static TCLIST *searchfiles(TCMPOOL *mpool, const char *expr, const char *order,
     tcmapput2(file, "date", datestrsimple(numbuf));
     tcmapprintf(file, "size", "%lld", (long long)size);
     const char *type = mimetype(name);
+    if(!type) type = "application/octet-stream";
     tcmapput2(file, "type", type);
     tcmapput2(file, "typename", mimetypename(type));
     if(thum && tcstrfwm(type, "image/") && !strchr(type, '+'))
